package cz.cvut.fel.pr2.cv8;

import javax.swing.*;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

/**
 * @author Ondrej Hrstka (ondrej.hrstka at agents.fel.cvut.cz)
 */
public class SimpleGUI implements Observer {

    private final CounterModel model;

    private JLabel counterValueLabel;
    private CountDownAction countDownAction;
    private CountUpAction countUpAction;


    public SimpleGUI(CounterModel model) {
        this.model = model;
        model.addObserver(this);

        initActions();
        final JFrame frame = createJFrame();

        model.notifyObservers(); //force view/controller to reload model

        JOptionPane.showMessageDialog(frame, "Try also popup menu using mouse right click.");
    }

    private JFrame createJFrame() {
        JPanel panel = createJPanel();

        JFrame frame = new JFrame("Simple Counter");
        frame.setContentPane(panel);
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        frame.setJMenuBar(createJMenuBar());
        frame.setVisible(true);
        frame.pack();

        return frame;
    }

    private JMenuBar createJMenuBar() {
        JMenu menu = new JMenu("Actions");
        menu.add(countUpAction);
        menu.add(countDownAction);

        //Or without action
        JMenuItem exitItem = new JMenuItem("Exit");
        exitItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        menu.add(exitItem);


        JMenuBar out = new JMenuBar();
        out.add(menu);

        return out;
    }

    private void initActions() {
        countUpAction = new CountUpAction(model);
        countDownAction = new CountDownAction(model);
    }

    private JPanel createJPanel() {
        initCounterValueLabel();

        JPanel panel = new JPanel(new BorderLayout());

        panel.add(new JButton(countUpAction), BorderLayout.NORTH);
        panel.add(counterValueLabel, BorderLayout.CENTER);
        panel.add(new JButton(countDownAction), BorderLayout.SOUTH);

        JPopupMenu popupMenu = createPopupMenu();
        panel.setComponentPopupMenu(popupMenu);

        return panel;
    }

    private JPopupMenu createPopupMenu() {
        JPopupMenu out = new JPopupMenu();
        out.add(countUpAction);
        out.add(countDownAction);
        return out;
    }

    private void initCounterValueLabel() {
        counterValueLabel = new JLabel();
        counterValueLabel.setText("X");
        counterValueLabel.setFont(counterValueLabel.getFont().deriveFont(30f)); //set bigger font
        counterValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
    }

    @Override
    public void update(Observable o, Object arg) {
        counterValueLabel.setText(String.valueOf(model.getValue()));
    }

    public static void main(String[] args) {
        CounterModel model = new CounterModel();
        new SimpleGUI(model);
    }
}
